/*
 * An implementation of Rabin-Karp matching for this problem: http://vn.spoj.com/problems/SUBSTR/
 * For more information about this implementation see  http://www.giaithuatlaptrinh.com/?p=290
 *  Created on: Jul 19, 2015
 *      Author: hunglv
 */
#include<stdio.h>
#include<iostream>
#include<stdlib.h>
#include<math.h>
#include<stack>
#include<string.h>

using namespace std;
#define max(x, y) (((x) < (y)) ? (y) : (x))
#define MAX 1000007
#define PRIME 1
#define COMPOSITE 0

typedef unsigned long long int LLU;
string T, P;


int miller_rabin_testing(int acc, LLU N);
LLU gcd(LLU a, LLU b);
int witness(LLU a,  LLU N);
void decompose(LLU p, int *k, LLU *m);
LLU random64();
LLU mod_mul(LLU a, LLU b, LLU p);
LLU mod_power(LLU a, LLU b, LLU p);
void RabinKarpMatcher(int n, int m);
LLU random_prime(LLU N);

int main() {
    cin >> T >> P;

    int n=T.size(), m=P.size();
    T = " " + T;
    P = " " + P ;
    RabinKarpMatcher(n,m);
}


void RabinKarpMatcher(int n, int m){
	LLU base = (LLU) 26; // the alphabet is {a,b,c,d,...,z}
	LLU q = random_prime((LLU)(1<<30) - 1);// getting a random prime number, assume that m^2 log m <= 2^(30)
	LLU p = 0;
	LLU Ts[n-m+2];
	LLU sig = mod_power(base,m-1,q);
	Ts[1] = 0;
	int i = 0;
	for(i = 1; i <=m; i++){
		p = (P[i]-'a' + ((base*p)%q))%q;
		Ts[1] = (T[i]-'a'+ ((base*Ts[1])%q))%q;
	}
	for(i = 1; i <= n-m+1; i++){
		if(Ts[i] == p){
			printf("%d ",i);
		}
		Ts[i+1] =  ((base*((Ts[i]  - ((sig*(T[i]-'a')) %q) + q) %q))%q + T[i+m]-'a')%q;
	}
}

LLU random_prime(LLU N){
	int rand = random64()%(N-2)+2;
	while(miller_rabin_testing(10, rand) != PRIME){
		rand = random64()%(N-2)+2;
	}
	return rand;
}

/*
 * Miller-Rabin Primality testing algorithm
 * See //www.giaithuatlaptrinh.com/?p=278 for details
 */

int miller_rabin_testing(int acc, LLU N){
	LLU a = 0;
	for(int i = 0; i <= acc; i++){
		a = random64()%(N-2)+ 2;
		if(gcd(a,N) != 1){
			return COMPOSITE;
		} else if (mod_power(a,N-1,N) != 1){
			return COMPOSITE;
		}else {
			if (witness(a,N) == COMPOSITE){
				return COMPOSITE;
			}
		}
	}
	return PRIME;
}

LLU gcd(LLU a, LLU b){
	LLU tmp;
	while  (b != 0){
		tmp  = b;
		b = a % b;
		a = tmp;
	}
	return a;
}

LLU mod_power(LLU a, LLU b, LLU p){
	if(b == 0) return 1;
	else if (b == 1){
		return a %p;
	} else {
		LLU x = mod_power(a, b/2, p);
		if((b& 1) == 0){ // b is even
			return mod_mul(x,x,p); // return  (x*x)%p;
		} else {
			return (mod_mul(mod_mul(x,x,p), a, p));// return (x*x*a)%p;
		}
	}
}
/*
 * this function compute (a*b) \mod p using (\log_2a + 1) space
 */

LLU mod_mul(LLU a, LLU b, LLU p){
	if (b == 1){
		return a %p;
	} else {
		LLU x = mod_mul(a, b/2, p);
		if((b& 1) == 0){ // b is even
			return (x+x)%p;
		} else {
			return (((x+x)%p)+a)%p;
		}
	}
}


int witness(LLU a,  LLU N){
	LLU m;
	int k;
	decompose(N-1,&k,&m);
	LLU B[k+1];
	B[0] = mod_power(a,m,N);
	if(B[0] == 1){
		return PRIME;
	} else {
		int i = 1;
		while(i <= k){
			B[i] =  mod_mul(B[i-1],B[i-1],N);// (B[i-1]*B[i-1])%N;
			if(B[i] == 1){
				if(B[i-1] == N-1){
					return PRIME;
				} else {
					return COMPOSITE;
				}
			}
			i++;
		}
	}
	return COMPOSITE;
}

void decompose(LLU p, int *k, LLU *m){
	int i = 0;
	while ((p& 1) == 0){ // p is even
		i++;
		p = p >> 1; // p = p /2
	}
	*k = i;
	*m = p;
}
/*
 * generating a random 64 bit number
 */
LLU random64(){
	LLU n  = 0;
	int l = rand()%(RAND_MAX-2)+2;
	int r = rand()%(RAND_MAX) + rand()%2;
	n = n | ((LLU)r);
	n = n << 32;
	n = n | ((LLU)l);
	return n;
}


